--
-- DT75
-- Specialization for DT75
--
-- @author  Aluha74rus
-- @date  17/09/16
-- @version 1.0 - Aluha74rus
--

DT75 = {};

function DT75.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Motorized, specializations);
end;

function DT75:load(savegame)
if self.isClient then
    self.samplePuskacStart = SoundUtil.loadSample(self.xmlFile, {}, "vehicle.puskacStartSound", nil, self.baseDirectory);
	self.sampleSwetSound         = SoundUtil.loadSample(self.xmlFile, {}, "vehicle.SwetSound", nil, self.baseDirectory);

    self.puskacEffects = {};
    DT75.loadPuskacEffects(self, self.xmlFile, self.puskacEffects);
    if table.getn(self.puskacEffects) == 0 then
        self.puskacEffects = nil;
    end
    self.startpuskacEffects = {};
    DT75.loadStartPuskacEffects(self, self.xmlFile, self.startpuskacEffects);
    if table.getn(self.startpuskacEffects) == 0 then
        self.startpuskacEffects = nil;
    end
end;	
	
	self.gearShifter = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.DT75.gearShifter#index"));
	
	local pedaltormozNode = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.DT75.pedaltormoz#index"));
    if pedaltormozNode ~= nil then
        self.pedaltormoz = {};
        self.pedaltormoz.node = pedaltormozNode;
        local x, y, z = Utils.getVectorFromString(getXMLString(self.xmlFile, "vehicle.DT75.pedaltormoz#minRot"));
        self.pedaltormoz.minRot = {};
        self.pedaltormoz.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.pedaltormoz.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.pedaltormoz.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));
        x, y, z = Utils.getVectorFromString(getXMLString(self.xmlFile, "vehicle.DT75.pedaltormoz#maxRot"));
        self.pedaltormoz.maxRot = {};
        self.pedaltormoz.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.pedaltormoz.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.pedaltormoz.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));
        self.pedaltormoz.rotTime = Utils.getNoNil(getXMLString(self.xmlFile, "vehicle.DT75.pedaltormoz#rotTime"), 2)*1000;
    end;	
	
	local seatNode = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.DT75.seat#index"));
	if seatNode ~= nil then
		self.seat = {};
		self.seat.node = seatNode;
		self.seat.wheelId = getXMLInt(self.xmlFile, "vehicle.DT75.seat#wheelReferenceId") or 1;
		self.seat.usagePercentage = (getXMLInt(self.xmlFile, "vehicle.DT75.seat#usagePercentage") or 30) / 100;
		self.seat.offsetY = getXMLFloat(self.xmlFile, "vehicle.DT75.seat#offsetY");
	end;
    self.PuskacDim = getXMLString(self.xmlFile, "vehicle.PuskacDim#animationName");
    self.Puskac = getXMLString(self.xmlFile, "vehicle.Puskac#animationName");
    self.puskacTurnedOnRotationNodes = Utils.loadRotationNodes(self.xmlFile, {}, "vehicle.turnedOnRotationNodes.turnedOnRotationNode", "puskac", self.components);

end;

function DT75:delete()
if self.isClient then
    SoundUtil.deleteSample(self.samplePuskacStart);
	SoundUtil.deleteSample(self.sampleSwetSound);
    if self.puskacEffects ~= nil then
        for _, effect1 in pairs(self.puskacEffects) do
            Utils.releaseSharedI3DFile(effect1.filename, self.baseDirectory, true);
        end
    end
    if self.startpuskacEffects ~= nil then
        for _, effect1 in pairs(self.startpuskacEffects) do
            Utils.releaseSharedI3DFile(effect1.filename, self.baseDirectory, true);
        end
    end
end;
end;

function DT75:loadPuskacEffects(xmlFile, puskacEffects)
    local i = 0;
    while true do
        local key = string.format("vehicle.puskacEffects.puskacEffect(%d)", i);
        if not hasXMLProperty(xmlFile, key) then
            break;
        end
        local linkNode = Utils.indexToObject(self.components, getXMLString(xmlFile, key.."#index"));
        local filename = getXMLString(xmlFile, key .. "#filename");
        if filename ~= nil and linkNode ~= nil then
            local i3dNode = Utils.loadSharedI3DFile(filename, self.baseDirectory, false, false, false);
            if i3dNode ~= 0 then
                local node = getChildAt(i3dNode, 0);
                if getHasShaderParameter(node, "param") then
                    local effect1 = {};
                    effect1.puskaceffectNode = node
                    effect1.node = linkNode
                    effect1.filename = filename;
                    link(effect1.node, effect1.puskaceffectNode);
                    setVisibility(effect1.puskaceffectNode, false);
                    delete(i3dNode);

                    effect1.minRpmColor = Utils.getVectorNFromString(Utils.getNoNil(getXMLString(xmlFile, key.."#minRpmColor"), "0 0 0 1"), 4);
                    effect1.maxRpmColor = Utils.getVectorNFromString(Utils.getNoNil(getXMLString(xmlFile, key.."#maxRpmColor"), "0.0384 0.0359 0.0627 2.0"), 4);
                    effect1.minRpmScale = Utils.getNoNil(getXMLFloat(xmlFile, key.."#minRpmScale"), 0.25);
                    effect1.maxRpmScale = Utils.getNoNil(getXMLFloat(xmlFile, key.."#maxRpmScale"), 0.95);

                    table.insert(puskacEffects, effect1);
                end
            end
        end
        i = i + 1;
    end
end

function DT75:loadStartPuskacEffects(xmlFile, startpuskacEffects)
    local i = 0;
    while true do
        local key = string.format("vehicle.startpuskacEffects.startpuskacEffect(%d)", i);
        if not hasXMLProperty(xmlFile, key) then
            break;
        end
        local linkNode = Utils.indexToObject(self.components, getXMLString(xmlFile, key.."#index"));
        local filename = getXMLString(xmlFile, key .. "#filename");
        if filename ~= nil and linkNode ~= nil then
            local i3dNode = Utils.loadSharedI3DFile(filename, self.baseDirectory, false, false, false);
            if i3dNode ~= 0 then
                local node = getChildAt(i3dNode, 0);
                if getHasShaderParameter(node, "param") then
                    local effect1 = {};
                    effect1.startpuskaceffectNode = node
                    effect1.node = linkNode
                    effect1.filename = filename;
                    link(effect1.node, effect1.startpuskaceffectNode);
                    setVisibility(effect1.startpuskaceffectNode, false);
                    delete(i3dNode);

                    effect1.minRpmColor = Utils.getVectorNFromString(Utils.getNoNil(getXMLString(xmlFile, key.."#minRpmColor"), "0 0 0 1"), 4);
                    effect1.maxRpmColor = Utils.getVectorNFromString(Utils.getNoNil(getXMLString(xmlFile, key.."#maxRpmColor"), "0.0384 0.0359 0.0627 2.0"), 4);
                    effect1.minRpmScale = Utils.getNoNil(getXMLFloat(xmlFile, key.."#minRpmScale"), 0.25);
                    effect1.maxRpmScale = Utils.getNoNil(getXMLFloat(xmlFile, key.."#maxRpmScale"), 0.95);

                    table.insert(startpuskacEffects, effect1);
                end
            end
        end
        i = i + 1;
    end
end

function DT75:readStream(streamId, connection)
end;

function DT75:writeStream(streamId, connection)
end;

function DT75:readUpdateStream(streamId, timestamp, connection)
end;

function DT75:writeUpdateStream(streamId, connection, dirtyMask)
end;

function DT75:mouseEvent(posX, posY, isDown, isUp, button)
end;

function DT75:keyEvent(unicode, sym, modifier, isDown)
end;

function DT75:update(dt)
    local PuskacDimEnableTimeStartDelay = 6000;
	local PuskacDimEnableTime = (self.motorStartTime - self.motorStartDuration) + PuskacDimEnableTimeStartDelay;
	local Duration = 4000;
	local DisableTime = PuskacDimEnableTime + Duration;
    local MotorPuskEnableTimeStartDelay = 4000;
	local MotorPuskEnableTime = (self.motorStartTime - self.motorStartDuration) + MotorPuskEnableTimeStartDelay;
    local PuskacSoundEnableTimeStartDelay = 12000;
	local PuskacSoundEnableTime = (self.motorStartTime - self.motorStartDuration) + PuskacSoundEnableTimeStartDelay;
    local PuskacEnableTimeStartDelay = 12000;
	local PuskacEnableTime = (self.motorStartTime - self.motorStartDuration) + PuskacEnableTimeStartDelay;
    Utils.updateRotationNodes(self, self.puskacTurnedOnRotationNodes, dt, g_currentMission.time <= PuskacEnableTime  and self.isMotorStarted);
	
	if self.isClient then
		if self.puskacEffects ~= nil then
		    if (g_currentMission.time <= PuskacSoundEnableTime  and self.isMotorStarted) then
	            for _, effect1 in pairs(self.puskacEffects) do
	                setVisibility(effect1.puskaceffectNode, true);
	
	                local color = effect1.minRpmColor;
	                setShaderParameter(effect1.puskaceffectNode, "puskacexhaustColor", color[1], color[2], color[3], color[4], false);
	            end
	            SoundUtil.playSample(self.samplePuskacStart, 0, 0, nil);
	            SoundUtil.stop3DSample(self.sampleMotor);
		    else
	            for _, effect1 in pairs(self.puskacEffects) do
	                setVisibility(effect1.puskaceffectNode, false);
	            end
	            SoundUtil.stopSample(self.samplePuskacStart);
		    end;
		end;
		if self.startpuskacEffects ~= nil then
		    if (g_currentMission.time <= DisableTime and g_currentMission.time >= PuskacDimEnableTime  and self.isMotorStarted) then
	            for _, effect1 in pairs(self.startpuskacEffects) do
	                setVisibility(effect1.startpuskaceffectNode, true);
	
	                local color = effect1.minRpmColor;
	                setShaderParameter(effect1.startpuskaceffectNode, "puskacexhaustColor", color[1], color[2], color[3], color[4], false);
	            end
		    else
	            for _, effect1 in pairs(self.startpuskacEffects) do
	                setVisibility(effect1.startpuskaceffectNode, false);
	            end
		    end;
		end;
		if self.Puskac ~= nil and self.playAnimation ~= nil then
		    if (g_currentMission.time <= PuskacEnableTime  and self.isMotorStarted) then
                self:playAnimation(self.Puskac, 1, nil, true);
		    else
                self:playAnimation(self.Puskac, -1, nil, true);
		    end;
		end;
		if self.PuskacDim ~= nil and self.playAnimation ~= nil then
		    if (g_currentMission.time <= MotorPuskEnableTime  and self.isMotorStarted) then
                self:playAnimation(self.PuskacDim, 1, nil, true);
		    else
                self:playAnimation(self.PuskacDim, -1, nil, true);
		    end;
		end;	
		if self:getIsAnimationPlaying("SwetWerh") then
			if self.animations["SwetWerh"].currentSpeed > 0 then
				if (not SoundUtil.isSamplePlaying(self.SwetSound) and not SoundUtil.isSamplePlaying(self.SwetSound)) and self:getIsActiveForSound() then
					SoundUtil.playSample(self.sampleSwetSound,1,1,nil);
				end;
			elseif self.animations["SwetWerh"].currentSpeed < 0 then
				if (not SoundUtil.isSamplePlaying(self.SwetSound) and not SoundUtil.isSamplePlaying(self.SwetSound)) and self:getIsActiveForSound() then
					SoundUtil.playSample(self.sampleSwetSound,1,1,nil);
				end;
			end;
		end;	
	end;
	if self:getIsActive() and self.isMotorStarted and self.isClient then
        local speed = self.lastSpeed*3600;
        if self.gearShifter ~= nil then    
            if speed < 0.1 then
                setRotation(self.gearShifter, Utils.degToRad(0),   Utils.degToRad(0), Utils.degToRad(0));
            elseif speed >= 0.1 and speed < 4 then
                setRotation(self.gearShifter, Utils.degToRad(10), Utils.degToRad(-10), Utils.degToRad(10));
            elseif speed >= 4 and speed < 8 then
                setRotation(self.gearShifter, Utils.degToRad(-10), Utils.degToRad(0), Utils.degToRad(0));
            elseif speed >= 8 and speed < 12 then
                setRotation(self.gearShifter, Utils.degToRad(10),  Utils.degToRad(0), Utils.degToRad(0));
            elseif speed >= 12 and speed < 15 then
                setRotation(self.gearShifter, Utils.degToRad(10),  Utils.degToRad(10), Utils.degToRad(-10));
            elseif speed < -0.1 then
                setRotation(self.gearShifter, Utils.degToRad(0),   Utils.degToRad(0), Utils.degToRad(0));
            elseif speed <= -0.1 and speed > -8 then
                setRotation(self.gearShifter, Utils.degToRad(-17), Utils.degToRad(0), Utils.degToRad(-14));
            end;
        end;  
	end;
	    if self.pedaltormoz ~= nil then
		    local x, y, z = getRotation(self.pedaltormoz.node);
		    local rot = {x,y,z};
		    local newRot = Utils.getMovedLimitedValues(rot, self.pedaltormoz.maxRot, self.pedaltormoz.minRot, 3, self.pedaltormoz.rotTime, dt, self.brakeLightsVisibility);
		    setRotation(self.pedaltormoz.node, unpack(newRot));
	    end;
		if self.seat ~= nil and self.wheels[self.seat.wheelId] ~= nil then
			if self.seat.trans == nil then
				self.seat.trans = {getTranslation(self.seat.node)};
				
				if self.seat.offsetY ~= nil then
					self.seat.trans[2] = self.seat.trans[2] + self.seat.offsetY;
				end;
			end;
			local _, y, _ = getTranslation(self.wheels[self.seat.wheelId].repr);
			local newTrans = (y * self.seat.usagePercentage) + self.seat.trans[2];
			newTrans = math.floor(newTrans * 40^95 + 0.5) / 40^95;
			if newTrans ~= self.seat.lastTrans then
				setTranslation(self.seat.node, self.seat.trans[1], newTrans, self.seat.trans[3]);
				self.seat.lastTrans = newTrans;
			end;
		end;
end;

function DT75:updateTick(dt) 
    if self.isClient then
        if self:getIsMotorStarted() then
	        if self.puskacEffects ~= nil then
	            for _, effect1 in pairs(self.puskacEffects) do
	                local rpmScale = self.motor:getEqualizedMotorRpm() / self.motor:getMaxRpm();
	                local scale = Utils.lerp(effect1.minRpmScale, effect1.maxRpmScale, rpmScale);
	
	                local r = Utils.lerp(effect1.minRpmColor[1], effect1.maxRpmColor[1], rpmScale);
	                local g = Utils.lerp(effect1.minRpmColor[2], effect1.maxRpmColor[2], rpmScale);
	                local b = Utils.lerp(effect1.minRpmColor[3], effect1.maxRpmColor[3], rpmScale);
	                local a = Utils.lerp(effect1.minRpmColor[4], effect1.maxRpmColor[4], rpmScale);
	                setShaderParameter(effect1.puskaceffectNode, "puskacexhaustColor", r, g, b, a, false);
	            end
	        end
	        if self.startpuskacEffects ~= nil then
	            for _, effect1 in pairs(self.startpuskacEffects) do
	                local rpmScale = self.motor:getEqualizedMotorRpm() / self.motor:getMaxRpm();
	                local scale = Utils.lerp(effect1.minRpmScale, effect1.maxRpmScale, rpmScale);
	
	                local r = Utils.lerp(effect1.minRpmColor[1], effect1.maxRpmColor[1], rpmScale);
	                local g = Utils.lerp(effect1.minRpmColor[2], effect1.maxRpmColor[2], rpmScale);
	                local b = Utils.lerp(effect1.minRpmColor[3], effect1.maxRpmColor[3], rpmScale);
	                local a = Utils.lerp(effect1.minRpmColor[4], effect1.maxRpmColor[4], rpmScale);
	                setShaderParameter(effect1.startpuskaceffectNode, "puskacexhaustColor", r, g, b, a, false);
	            end
	        end
	    end
	end
end;
function DT75:draw()
end;

function DT75:onLeave()
end;

function DT75:onEnter()
end;